#!/usr/bin/env python3
'''
This file is part of HTD (HayzenTech Database)

Copyright (C) 2011 - 2013 Andrew Hayzen

HTD is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

HTD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with HTD. If not, see <http://www.gnu.org/licenses/>

This document conforms to the PEP 8 Style Guide
'''


#----------------------------------------------------- Event Management for HTD
class EventType(object):
    """ Event types """
    BEFORE_LOCK = 1
    BEFORE_RUN = 2
    ERROR = 3
    AFTER_RUN = 4
    AFTER_UNLOCK = 5
    ALL_EVENTS = 6
    ALL_SIGNALS = 7


class Event(object):
    """ Class that emits and connects event """

    # Load events for ALL_SIGNALS
    signals = {7: {1: [], 2: [], 3: [], 4: [], 5: [], 6: []}}

    @classmethod
    def connect(cls, event, signal, func):
        """ Connect function to the event->signal """

        if signal not in cls.signals:
            cls.signals[signal] = {6: []}

        if event not in cls.signals[signal]:
            cls.signals[signal][event] = []

        if func not in cls.signals[signal][event]:
            cls.signals[signal][event].append(func)

    @classmethod
    def disconnect(cls, event, signal, func):
        """ Disconnect function from the event->signal """

        try:
            if func in cls.signals[signal][event]:
                cls.signals[signal][event].remove(func)
        except KeyError:
            raise KeyError("Could not connect func as no event or signal "
                           "existed (%s, %s)" % event, signal)

        return

    @classmethod
    def emit(cls, idn, event, signal, args=[], kwargs={}):
        """ Emit a event->signal to the relevant functions """

        for i in cls.signals[7][event]:  # Run ALL_SIGNALS
            i(idn, event, signal, args, kwargs)

        for i in cls.signals[7][6]:  # Run ALL_EVENTS ALL_SIGNALS
            i(idn, event, signal, args, kwargs)

        if signal in cls.signals:
            for i in cls.signals[signal][6]:  # Run ALL_EVENTS
                i(idn, event, signal, args, kwargs)

            if event in cls.signals[signal]:
                for i in cls.signals[signal][event]:  # Run Normal
                    i(idn, event, signal, args, kwargs)
