#!/usr/bin/env python3
'''
This file is part of HTD (HayzenTech Database)

Copyright (C) 2011 - 2013 Andrew Hayzen

HTD is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

HTD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with HTD. If not, see <http://www.gnu.org/licenses/>

This document conforms to the PEP 8 Style Guide
'''


class DatabaseError(Exception):
    pass


class AuthenticationError(DatabaseError):
    pass


class ConnectionError(DatabaseError):
    pass


class ConcurrencyError(DatabaseError):
    pass


class IntegrityError(DatabaseError):
    pass


class VersionError(DatabaseError):
    pass


def perm(level, access):
    """ Run access permission check """

    perms = {"r": "read", "o": "overwrite", "a": "append",
             "d": "delete", "x": "admin"}

    access = [i for i in level if i not in access]

    if len(access) == 0:
        return True

    err = ", ".join(['(%s - %s)' % (i, perms[i]) for i in access])
    err = 'User doesn\'t have to permission [%s]' % err

    raise AuthenticationError(err)
