#!/usr/bin/env python3
'''
This file is part of HTD (HayzenTech Database)

Copyright (C) 2011 - 2013 Andrew Hayzen

HTD is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

HTD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with HTD. If not, see <http://www.gnu.org/licenses/>

This document conforms to the PEP 8 Style Guide
'''

from copy import copy


#-------------------------------------------------- Progress management for HTD
class ProgressItem():
    """ A progression Item """
    counter = 0

    def __init__(self, thread, length, text):
        self.thread = thread
        self.length = length
        self.text = text

    def count(self, amount=1):
        """ Increase the counter by amount """

        self.counter += amount


class Progress():
    """ Progress management """
    tasks = {}
    threads = {}

    @classmethod
    def add(cls, idn, progress):
        """ Add new progress item """
        if not isinstance(progress, ProgressItem):
            raise ValueError("Must be a ProgressItem instance")

        cls.tasks[idn] = progress

        if progress.thread not in cls.threads:
            cls.threads[progress.thread] = []

        cls.threads[progress.thread].append(progress)

    @classmethod
    def get(cls, idn):
        """ Get the task with idn """

        return cls.tasks[idn]

    @classmethod
    def progress(cls):
        """ Return all running tasks """

        return [job for job in copy(cls.tasks.values())]

    @classmethod
    def remove(cls, idn):
        """ Remove task with idn """

        try:
            task = cls.tasks[idn]
            cls.threads[task.thread].remove(task)
        except KeyError:
            pass

        try:
            del cls.tasks[idn]
        except KeyError:
            pass

    @classmethod
    def thread_progress(cls, thread):
        """ Return the running tasks for a specific thread """

        if thread not in cls.threads:
            raise KeyError("Thread does not exist")

        return [{"length":job.length, "count":job.counter, "name":job.text,
                "fraction":job.counter / job.length} for job in
                copy(cls.threads[thread])]
