#!/usr/bin/env python3
'''
This file is part of HTD (HayzenTech Database)

Copyright (C) 2011 - 2013 Andrew Hayzen

HTD is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

HTD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with HTD. If not, see <http://www.gnu.org/licenses/>

This document conforms to the PEP 8 Style Guide

Data Filter to import/export raw data
Also loads default file handlers
get_line alog and locking classes
'''

from .. import Config
from time import sleep
from threading import current_thread


class Locking(object):
    def __init__(self):
        self._confirm_funcs = []
        self._idn = None
        self._locked = False

    def lock(self, exclusive=False, idn=None):
        if idn is not None:
            while self._idn is not None:
                sleep(0.003)

            self._idn = idn

        if idn is None and current_thread() == self._idn:
            return

        while 1:
            try:
                if exclusive:
                    self._file.lock_writing()
                else:
                    self._file.lock_reading()
                break
            except IOError:
                pass

            sleep(0.003)

        self._locked = True

        for func in self._confirm_funcs:
            func()

    @property
    def locked(self):
        return self._locked

    def unlock(self, idn=None):
        if idn is None and current_thread() == self._idn:
            return

        sync = Config.force_sync and self._file.locked

        for file in self.files:
            file.flush(fsync=sync)  # Flush files with optional Fsync

        try:
            self._file.unlock()
        except IOError:
            pass

        self._locked = False


# Import default file handlers
from . import file_var
from . import file_fix
