#!/usr/bin/env python3
'''
This file is part of HTD (HayzenTech Database)

Copyright (C) 2011 - 2013 Andrew Hayzen

HTD is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

HTD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with HTD. If not, see <http://www.gnu.org/licenses/>

This document conforms to the PEP 8 Style Guide
'''

from ... import Register
from ..._base import location_is_file
from ..base import BaseFile
from .file import FileWriter
from ..header.inline_json import JSONHeader
from os.path import exists
from ..search import BinarySearch


def build_data(data, file, index, atts, esc, new=False):
    string = []
    offset = 0

    if not new:
        current = file[index]

    for att in atts:
        if data[atts[att]["name"]] is None:
            if new:
                d = b""
            else:
                d = current[offset:offset + atts[att]["size"] + 1]
        else:
            d = b"".join([data[atts[att]["name"]], b"."])

        string.append(d.ljust(atts[att]["size"] + 1, esc))

        offset += atts[att]["size"] + 1

    return b"".join(string)


def read_data(data, atts, exclude=[]):
    output = {}
    offset = 0

    # Read atts
    for att in atts:
        value = data[offset:atts[att]["size"] + offset + 1]

        if value == b"":
            return False

        value = value.rsplit(b".", 1)

        if len(value) == 1:
            value = None
        else:
            value = value[0]

        if atts[att]["name"] not in exclude:
            output[atts[att]["name"]] = value

        offset += atts[att]["size"] + 1

    return output


class FIXED_WIDTH_FILE(BaseFile, JSONHeader):
    escape_char = b"\x00"
    idn = b"\x05"
    persistent = True

    def __init__(self, location, access, header=None):
        # Set variables
        self.access = access
        self.files = []
        self.location = location

        # Load classes
        BaseFile.__init__(self)
        JSONHeader.__init__(self)

        self.build_data = build_data
        self.read_data = read_data
        self.searcher = BinarySearch

        # Check for file
        new_file = not exists(location)

        if new_file:
            if header is None:
                raise IOError("No HTD file found (%s)" % location)

            open(location, "wb").close()
            self.header = header  # Set and load header
        else:
            if header is not None:
                raise IOError('File already exists (%s)' % location)

            self.header_confirm()  # Load header

        self._file = FileWriter(location, self.header, new_file)

        self.files.append(self._file)
        self._header_update_funcs.append(self.calc_file_settings)
        self._confirm_funcs.append(self.header_confirm)


Register.register_fhandler(FIXED_WIDTH_FILE, location_is_file)
