#!/usr/bin/env python3
'''
This file is part of HTD (HayzenTech Database)

Copyright (C) 2011 - 2013 Andrew Hayzen

HTD is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

HTD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with HTD. If not, see <http://www.gnu.org/licenses/>

This document conforms to the PEP 8 Style Guide
'''

from ..._base.data import get_key_as_string


class BinarySearch(object):
    ordered = True

    @classmethod
    def add_line(cls, file, key, atts, error=True):
        line = cls.search(file, key, atts, new=True)

        if error and line["found"]:
            raise KeyError()

        return line

    @classmethod
    def get_line(cls, file, key, atts, new=False):
        line = cls.search(file, key, atts)

        if not line["found"]:
            raise KeyError()

        return {"index": line["index"], "data": line["data_line"]}

    @staticmethod
    def search(file, key, atts, new=False):
        key = get_key_as_string(key, atts)

        size = len(file) - 1
        upper = size
        lower = 0
        mid = file.get_top(size)

        if size == -1:  # Check table not empty
            return {"found": False, "index": 0}

        if new:  # Optimise last if inserting
            data_line = file.get_index(size)  # Load cline
            data_key = get_key_as_string(data_line, atts)

            if key > data_key:
                return {"found": False, "index": size + 1}

        while upper >= lower:  # Search for key
            data_line = file.get_index(mid)
            data_key = get_key_as_string(data_line, atts)  # Get key from data

            # Compare
            if data_key < key:
                mid, lower = file.get_right(lower, mid, upper)
            elif data_key > key:
                mid, upper = file.get_left(lower, mid, upper)
            else:
                break

        if lower > upper:
            mid = lower

        if data_key == key:
            return {"found": True, "index": mid, "data_line": data_line}
        else:
            return {"found": False, "index": mid}
